# coding=utf-8
from typing import Any

import os

from ut_pac.pac import Pac

TyArr = list[Any]
TyDic = dict[Any, Any]
TyPac = str
TyPath = str
TyAoPac = str | list[str]
TyAoPath = list[TyPath]

TnPath = None | TyPath


class AoPac:

    @staticmethod
    def sh_path_by_path_if_exists(
            aopac: TyAoPac, path: TyPath) -> TyPath:
        if not isinstance(aopac, (list, tuple)):
            _aopac = [aopac]
        else:
            _aopac = aopac
        _aopath : TyAoPath = []
        for _pac in _aopac:
            _path: TyPath = Pac.sh_path_by_path(_pac, path)
            if os.path.exists(_path):
                return _path
            _aopath.append(_path)
        msg = f"No path in array _aopath={_aopath} exists"
        raise Exception(msg)

    @staticmethod
    def sh_aopath_by_path(
            aopac: TyAoPac, path: TyPath) -> TyAoPath:
        if not isinstance(aopac, list):
            _aopac = [aopac]
        else:
            _aopac = aopac
        _aopath: TyAoPath = []
        for _pac in _aopac:
            _path = Pac.sh_path_by_path(_pac, path)
            _aopath.append(_path)
        return _aopath

    @classmethod
    def sh_aopath_by_paths(
            cls, aopac: TyAoPac, *paths: TyPath) -> TyAoPac:
        """
        Show the array of paths created by the array of packages
        and the given path extended by the given prefix.
        """
        _path: TyPath = os.path.join(*paths)
        _aopath: TyAoPath = cls.sh_aopath_by_path(aopac, _path)
        return _aopath



# coding=utf-8
from typing import Any

from ut_pac.pac import Pac

TyArr = list[Any]
TyObjPath = str
TyModPath = str
TyPacPath = str
TyModule = str
TyModName = str
TyPath = str

TnPath = None | TyPath


class Cls:

    @staticmethod
    def sh_pac_path(cls) -> TyPacPath:
        _module: TyModule = cls.__module__
        _a_module: TyArr = _module.split('.')
        # Exclude the module itself)
        # Extract package and subpackages from the module name
        _path: TyObjPath = '.'.join(_a_module[:-1])
        return _path

    @staticmethod
    def sh_mod_path(cls) -> TyModPath:
        _module: TyModule = cls.__module__
        return _module

    @staticmethod
    def sh_mod_name(cls) -> TyModName:
        _module: TyModule = cls.__module__
        _mod_name: TyModName = _module.rsplit('.', 1)[0]
        return _mod_name

    @staticmethod
    def sh_path_by_paths_if_exists(cls, *paths: TyPath) -> TyPath:
        """
        show path
        """
        _pac_path: TyPacPath = cls.sh_pac_path(cls)
        _path: TyPath = Pac.sh_path_by_path_if_exists(_pac_path, *paths)
        return _path

    @staticmethod
    def sh_path_by_path_if_exists(cls, path: TyPath) -> TyPath:
        """
        show path
        """
        _pac_path: TyPacPath = cls.sh_pac_path(cls)
        _path: TyPath = Pac.sh_path_by_path_if_exists(_pac_path, path)
        return _path



# coding=utf-8
import os
import importlib.resources

from typing import Any

TyArr = list[Any]
TyDic = dict[Any, Any]
TyMod = str
TyPac = str
TyPacPath = str
TyPath = str


class Pac:

    @staticmethod
    def sh_path(pac_path: TyPacPath) -> TyPath:
        _path: TyPath = str(importlib.resources.files(pac_path))
        return _path

    @classmethod
    def sh_path_by_paths_if_exists(
            cls, pac_path: TyPacPath, *paths: TyPath) -> TyPath:
        # def sh_path_by_path_and_prefix(
        """
        show directory
        """
        _path: TyPath = os.path.join(*paths)
        _path = cls.sh_path_by_path_if_exists(pac_path, _path)
        return _path

    @classmethod
    def sh_path_by_path_if_exists(
            cls, pac_path: TyPacPath, path: TyPath) -> TyPath:
        """ show directory
        """
        _path: TyPath = cls.sh_path_by_path(pac_path, path)
        if not _path:
            msg = f"path={path} does not exist in pac_path={pac_path}"
            raise Exception(msg)
        if os.path.exists(_path):
            return _path
        msg = f"path={_path} for pac_path={pac_path} does not exist"
        raise Exception(msg)

    @classmethod
    def sh_path_by_paths(
            cls, pac_path: TyPacPath, *paths: TyPath) -> TyPath:
        # def sh_path_by_path_and_prefix(
        """
        show directory
        """
        _path: TyPath = os.path.join(*paths)
        _path = cls.sh_path_by_path(pac_path, _path)
        return _path

    @staticmethod
    def sh_path_by_path(
            pac_path: TyPacPath, path: TyPath) -> TyPath:
        """ show directory
        """
        _path: TyPath = str(importlib.resources.files(pac_path).joinpath(path))
        return _path



# coding=utf-8

from typing import Any
TyArr = list[Any]
TyMod = str
TyPac = str
TyPointPath = str
TyStr = str

TnPac = None | TyPac


class Fnc:

    @staticmethod
    def sh_full_name(fnc) -> TyPointPath:
        # def sh_pacmod_name(fnc) -> TyPacModName:
        """
        show full qualified name of object
        """
        _class = fnc.__class__
        _module: TyPointPath = _class.__module__
        _qualname: TyPointPath = _class.__qualname__
        # Avoid prefixing built-in types
        if _module == "builtins":
            return _qualname
        return f"{_module}.{_qualname}"

    # @staticmethod
    # def sh_mod(fnc) -> TyMod:
    #     # def sh_pacmod_name(fnc) -> TyPacModName:
    #     """
    #     show module name of function
    #     """
    #     _mod: TyMod = fnc.__module__
    #     return _mod

    @staticmethod
    def sh_pac(fnc) -> TyPac:
        # def sh_pac_name(fnc) -> TyModName:
        """
        show module name of function
        """
        _module: TyPac = fnc.__module__
        _a_pac: TyArr = _module.split('.')
        _pac: TyPac = '.'.join(_a_pac[-1])
        return _pac

    @staticmethod
    def sh_pac_first(fnc) -> TnPac:
        # def sh_pac_name(fnc) -> TyModName:
        """
        show module name of function
        """
        _module = fnc.__module__
        _pac: TnPac = _module.split('.', 1)
        return _pac

    @staticmethod
    def sh_pac_last(fnc) -> TnPac:
        # def sh_pac_name(fnc) -> TyModName:
        """
        show module name of function
        """
        _module = fnc.__module__
        _pac: TnPac = _module.rsplit('.', 1)
        return _pac

    # @staticmethod
    # def sh_cls_name(fnc) -> TyClsName:
    #     """
    #     show class name of function
    #     """
    #     if hasattr(fnc, '__qualname__'):
    #         _cls_name: TyClsName = fnc.__qualname__.split('.', 1)
    #     return _cls_name

    # @classmethod
    # def sh_fnc_name(cls, fnc) -> TyFncName:
    #     """
    #     show class name of function
    #     """
    #     _pac: TyPac = cls.sh_pac(cls)
    #     _cls_name = cls.sh_cls_name(fnc)
    #     _fnc_name = '.'.join([_pac, _cls_name])
    #     return _fnc_name
